@@ -56,19 +56,19 @@ static char *name = "";
 static char *strippedname = "";
 static char description[255];
 static char cloneurl[1024];
-static int haslicense, hasreadme, hassubmodules;
+static char *submodules;
+static char *licensefiles[] = { "HEAD:LICENSE", "HEAD:LICENSE.md", "HEAD:COPYING" };
+static char *license;
+static char *readmefiles[] = { "HEAD:README", "HEAD:README.md" };
+static char *readme;
 static long long nlogcommits = -1; /* < 0 indicates not used */
 
 /* cache */
 static git_oid lastoid;
-static char lastoidstr[GIT_OID_HEXSZ + 2]; /* id + newline + nul byte */
+static char lastoidstr[GIT_OID_HEXSZ + 2]; /* id + newline + NUL byte */
 static FILE *rcachefp, *wcachefp;
 static const char *cachefile;
 
-#ifndef USE_PLEDGE
-#define pledge(p1,p2) 0
-#endif
-
 void
 joinpath(char *buf, size_t bufsiz, const char *path, const char *path2)
 {
@@ -96,6 +96,7 @@ commitinfo_getstats(struct commitinfo *ci)
 {
 	struct deltainfo *di;
 	git_diff_options opts;
+	git_diff_find_options fopts;
 	const git_diff_delta *delta;
 	const git_diff_hunk *hunk;
 	const git_diff_line *line;
@@ -113,10 +114,20 @@ commitinfo_getstats(struct commitinfo *ci)
 	}
 
 	git_diff_init_options(&opts, GIT_DIFF_OPTIONS_VERSION);
-	opts.flags |= GIT_DIFF_DISABLE_PATHSPEC_MATCH;
+	opts.flags |= GIT_DIFF_DISABLE_PATHSPEC_MATCH |
+	              GIT_DIFF_IGNORE_SUBMODULES |
+		      GIT_DIFF_INCLUDE_TYPECHANGE;
 	if (git_diff_tree_to_tree(&(ci->diff), repo, ci->parent_tree, ci->commit_tree, &opts))
 		goto err;
 
+	if (git_diff_find_init_options(&fopts, GIT_DIFF_FIND_OPTIONS_VERSION))
+		goto err;
+	/* find renames and copies, exact matches (no heuristic) for renames. */
+	fopts.flags |= GIT_DIFF_FIND_RENAMES | GIT_DIFF_FIND_COPIES |
+	               GIT_DIFF_FIND_EXACT_MATCH_ONLY;
+	if (git_diff_find_similar(ci->diff, &fopts))
+		goto err;
+
 	ndeltas = git_diff_num_deltas(ci->diff);
 	if (ndeltas && !(ci->deltas = calloc(ndeltas, sizeof(struct deltainfo *))))
 		err(1, "calloc");
@@ -124,6 +135,7 @@ commitinfo_getstats(struct commitinfo *ci)
 	for (i = 0; i < ndeltas; i++) {
 		if (git_patch_from_diff(&patch, ci->diff, i))
 			goto err;
+
 		if (!(di = calloc(1, sizeof(struct deltainfo))))
 			err(1, "calloc");
 		di->patch = patch;
@@ -362,12 +374,15 @@ writeheader(FILE *fp, const char *title)
 	fprintf(fp, "<a href=\"%slog.html\">Log</a> | ", relpath);
 	fprintf(fp, "<a href=\"%sfiles.html\">Files</a> | ", relpath);
 	fprintf(fp, "<a href=\"%srefs.html\">Refs</a>", relpath);
-	if (hassubmodules)
-		fprintf(fp, " | <a href=\"%sfile/.gitmodules.html\">Submodules</a>", relpath);
-	if (hasreadme)
-		fprintf(fp, " | <a href=\"%sfile/README.html\">README</a>", relpath);
-	if (haslicense)
-		fprintf(fp, " | <a href=\"%sfile/LICENSE.html\">LICENSE</a>", relpath);
+	if (submodules)
+		fprintf(fp, " | <a href=\"%sfile/%s.html\">Submodules</a>",
+		        relpath, submodules);
+	if (readme)
+		fprintf(fp, " | <a href=\"%sfile/%s.html\">README</a>",
+		        relpath, readme);
+	if (license)
+		fprintf(fp, " | <a href=\"%sfile/%s.html\">LICENSE</a>",
+		        relpath, license);
 	fputs("</td></tr></table>\n<hr/>\n<div id=\"content\">\n", fp);
 }
 
@@ -446,6 +461,7 @@ printshowfile(FILE *fp, struct commitinfo *ci)
 	git_patch *patch;
 	size_t nhunks, nhunklines, changed, add, del, total, i, j, k;
 	char linestr[80];
+	int c;
 
 	printcommit(fp, ci);
 
@@ -464,7 +480,22 @@ printshowfile(FILE *fp, struct commitinfo *ci)
 	fputs("<b>Diffstat:</b>\n<table>", fp);
 	for (i = 0; i < ci->ndeltas; i++) {
 		delta = git_patch_get_delta(ci->deltas[i]->patch);
-		fprintf(fp, "<tr><td><a href=\"#h%zu\">", i);
+
+		switch (delta->status) {
+		case GIT_DELTA_ADDED:      c = 'A'; break;
+		case GIT_DELTA_COPIED:     c = 'C'; break;
+		case GIT_DELTA_DELETED:    c = 'D'; break;
+		case GIT_DELTA_MODIFIED:   c = 'M'; break;
+		case GIT_DELTA_RENAMED:    c = 'R'; break;
+		case GIT_DELTA_TYPECHANGE: c = 'T'; break;
+		default:                   c = ' '; break;
+		}
+		if (c == ' ')
+			fprintf(fp, "<tr><td>%c", c);
+		else
+			fprintf(fp, "<tr><td class=\"%c\">%c", c, c);
+
+		fprintf(fp, "</td><td><a href=\"#h%zu\">", i);
 		xmlencode(fp, delta->old_file.path, strlen(delta->old_file.path));
 		if (strcmp(delta->old_file.path, delta->new_file.path)) {
 			fputs(" -&gt; ", fp);
@@ -575,7 +606,6 @@ writelog(FILE *fp, const git_oid *oid)
 
 	git_revwalk_new(&w, repo);
 	git_revwalk_push(w, oid);
-	git_revwalk_sorting(w, GIT_SORT_TIME);
 	git_revwalk_simplify_first_parent(w);
 
 	while (!git_revwalk_next(&id, w)) {
@@ -606,6 +636,10 @@ writelog(FILE *fp, const git_oid *oid)
 		} else if (nlogcommits > 0) {
 			writelogline(fp, ci);
 			nlogcommits--;
+			if (!nlogcommits && ci->parentoid[0])
+				fputs("<tr><td></td><td colspan=\"5\">"
+				      "More commits remaining [...]</td>"
+				      "</tr>\n", fp);
 		}
 
 		if (cachefile)
@@ -701,7 +735,6 @@ writeatom(FILE *fp)
 
 	git_revwalk_new(&w, repo);
 	git_revwalk_push_head(w);
-	git_revwalk_sorting(w, GIT_SORT_TIME);
 	git_revwalk_simplify_first_parent(w);
 
 	for (i = 0; i < m && !git_revwalk_next(&id, w); i++) {
@@ -1010,7 +1043,7 @@ err:
 void
 usage(char *argv0)
 {
-	fprintf(stderr, "%s [-c cachefile] [-l commits] repodir\n", argv0);
+	fprintf(stderr, "%s [-c cachefile | -l commits] repodir\n", argv0);
 	exit(1);
 }
 
@@ -1019,7 +1052,6 @@ main(int argc, char *argv[])
 {
 	git_object *obj = NULL;
 	const git_oid *head = NULL;
-	const git_error *e = NULL;
 	mode_t mask;
 	FILE *fp, *fpread;
 	char path[PATH_MAX], repodirabs[PATH_MAX + 1], *p;
@@ -1042,10 +1074,7 @@ main(int argc, char *argv[])
 			errno = 0;
 			nlogcommits = strtoll(argv[++i], &p, 10);
 			if (argv[i][0] == '\0' || *p != '\0' ||
-			    nlogcommits <= 0)
-				usage(argv[0]);
-			if (errno == ERANGE && (nlogcommits == LLONG_MAX ||
-			    nlogcommits == LLONG_MIN))
+			    nlogcommits <= 0 || errno)
 				usage(argv[0]);
 		}
 	}
@@ -1057,6 +1086,7 @@ main(int argc, char *argv[])
 
 	git_libgit2_init();
 
+#ifdef __OpenBSD__
 	if (cachefile) {
 		if (pledge("stdio rpath wpath cpath fattr", NULL) == -1)
 			err(1, "pledge");
@@ -1064,11 +1094,11 @@ main(int argc, char *argv[])
 		if (pledge("stdio rpath wpath cpath", NULL) == -1)
 			err(1, "pledge");
 	}
+#endif
 
 	if (git_repository_open_ext(&repo, repodir,
 		GIT_REPOSITORY_OPEN_NO_SEARCH, NULL) < 0) {
-		e = giterr_last();
-		fprintf(stderr, "%s: %s\n", argv[0], e->message);
+		fprintf(stderr, "%s: cannot open repository\n", argv[0]);
 		return 1;
 	}
 
@@ -1116,17 +1146,24 @@ main(int argc, char *argv[])
 	}
 
 	/* check LICENSE */
-	haslicense = (!git_revparse_single(&obj, repo, "HEAD:LICENSE") &&
-		git_object_type(obj) == GIT_OBJ_BLOB);
-	git_object_free(obj);
+	for (i = 0; i < sizeof(licensefiles) / sizeof(*licensefiles) && !license; i++) {
+		if (!git_revparse_single(&obj, repo, licensefiles[i]) &&
+		    git_object_type(obj) == GIT_OBJ_BLOB)
+			license = licensefiles[i] + strlen("HEAD:");
+		git_object_free(obj);
+	}
 
 	/* check README */
-	hasreadme = (!git_revparse_single(&obj, repo, "HEAD:README") &&
-		git_object_type(obj) == GIT_OBJ_BLOB);
-	git_object_free(obj);
+	for (i = 0; i < sizeof(readmefiles) / sizeof(*readmefiles) && !readme; i++) {
+		if (!git_revparse_single(&obj, repo, readmefiles[i]) &&
+		    git_object_type(obj) == GIT_OBJ_BLOB)
+			readme = readmefiles[i] + strlen("HEAD:");
+		git_object_free(obj);
+	}
 
-	hassubmodules = (!git_revparse_single(&obj, repo, "HEAD:.gitmodules") &&
-		git_object_type(obj) == GIT_OBJ_BLOB);
+	if (!git_revparse_single(&obj, repo, "HEAD:.gitmodules") &&
+	    git_object_type(obj) == GIT_OBJ_BLOB)
+		submodules = ".gitmodules";
 	git_object_free(obj);
 
 	/* log for HEAD */
